/**
 * Test functionality of class TuringMachine
 *
 */
public class Test {

	public static void main(String[] args) {
		Test t = new Test();
		t.runNondeterministicTest();
	}
		
	// TM which accepts language L = (a|b)*bb(a|b)*, i.e. any words which contain 'bb'
	// Left and right boundary are marked with symbols '#' and '$', TM terminates with
	// recognized substring 'bb' replaced by 'xx' if the word is accepted
	private void runNondeterministicTest() {
		// Create TM with 3 states and 5 symbols
		TuringMachine tm = new TuringMachine(3, 5);
		// Get tape for later writing of input
		TMTape tape = tm.getTape();
		// Add 3 states with S0 as initial state and S2 as final state
		tm.addState("S0", true, false);
		tm.addState("S1", false, false);
		tm.addState("S2", false, true);
		// Add 5 symbols (with 2 special symbols for left and right boundary)
		TMSymbol lbound = tm.addSymbol(TMSymbol.BOUND);
		TMSymbol a = tm.addSymbol("a");
		TMSymbol b = tm.addSymbol("b");
		TMSymbol x = tm.addSymbol("x");
		TMSymbol rbound = tm.addSymbol("$");
		
		// Add transitions: <origin>, <input>, <target>, <output>, <direction>
		
		// Move head in a nondeterministic way across the word
		tm.addTransition("S0", "a", "S0", "a", TuringMachine.RIGHT);
		tm.addTransition("S0", "b", "S0", "b", TuringMachine.RIGHT);
		tm.addTransition("S0", "a", "S0", "a", TuringMachine.LEFT);
		tm.addTransition("S0", "b", "S0", "b", TuringMachine.LEFT);
		tm.addTransition("S0", "$", "S0", "$", TuringMachine.LEFT);
		tm.addTransition("S0", TMSymbol.BOUND, "S0", TMSymbol.BOUND, TuringMachine.RIGHT);

		// Accept any 'b' and try to elongate substring in order to reach final state
		tm.addTransition("S0", "b", "S1", "x", TuringMachine.RIGHT);		
		tm.addTransition("S1", "b", "S2", "x", TuringMachine.HOLD);
		
		// Check word 'abaabb' which is in L
		System.out.println("Testing word 'abaabb'...");
		tape.write(0, lbound);
		tape.write(1, a);
		tape.write(2, b);
		tape.write(3, a);
		tape.write(4, a);
		tape.write(5, b);
		tape.write(6, b);
		tape.write(7, rbound);
		tm.init(0);
		if (tm.execute(true))
			System.out.println("-> TM accepted input!");
		else
			System.out.println("-> TM rejected input!");
		System.out.println("--------------------------------------------");
		
		// Check word 'abaaba' which is not in L
		System.out.println("Testing word 'abaaba'...");
		tape = tm.getTape();
		tape.write(0, lbound);
		tape.write(1, a);
		tape.write(2, b);
		tape.write(3, a);
		tape.write(4, a);
		tape.write(5, b);
		tape.write(6, a);
		tape.write(7, rbound);
		tm.init(0);
		if (tm.execute(true))
			System.out.println("-> TM accepted input!");
		else
			System.out.println("-> TM rejected input!");
	}
}
